#!perl -w
#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2008-2014 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################
# run as administrator => perllauncher RhapInstall.pl [RhapInstallRootDir]
# can be ran multiple times
#
use Win32::TieRegistry;
use File::Copy;

die "Environment variable TESTRTDIR not found. Exiting.\n\n" if(!defined($ENV{'TESTRTDIR'})); # NO TRANSLATION
$TESTRTDIR="$ENV{'TESTRTDIR'}";
$SCRIPTDIR="$TESTRTDIR/lib/scripts/";
$FILESDIR="$TESTRTDIR/targets/xml/attolcc/Rhapsody/";
require $SCRIPTDIR . "Fcts.pl";

my @RhapVERSIONS=qw( 8.0.5 7.6.1 7.6.0); #list of supported version (descending order) when not found in the registry
my @InstallDIRS=('C:\Program Files\IBM\Rational\Rhapsody' , 'C:\Program Files\Telelogic\Rhapsody','C:\Program Files(x86)\IBM\Rational\Rhapsody'); # when not found in the registry
my @regKeyToFind=("HKEY_LOCAL_MACHINE\\SOFTWARE\\Telelogic\\Rhapsody\\","HKEY_LOCAL_MACHINE\\SOFTWARE\\Wow6432Node\\Telelogic\\Rhapsody\\");
my $currentVersion;
sub Install()
{
    my $root;
    my $argVersion=$ARGV[1];
    my $InstallDir=$ARGV[0];
    $InstallDir =~s,[\\\/]$,,;

    if(-f $InstallDir."/rhapsody.exe"){
	    if(defined($ARGV[1])){
		$currentVersion=$ARGV[1];
	    } elsif($InstallDir=~m/[\\\/]([\d\.]+)[\\\/]/){
	    	$currentVersion=$1;
	    }
    } elsif(-f "$InstallDir/$argVersion/rhapsody.exe"){
		$InstallDir.="/$argVersion";
  		$currentVersion=$argVersion;
    } else {
	my $mnp;
	foreach (@regKeyToFind){
		$KeyToFind=$_;
		$mnp=$Registry->{$KeyToFind};
		last if($mnp);
	}
	if($mnp){
	  foreach ($mnp->MemberNames) {
	      if(/^[678]\./) { #takes into account only version 6,7,8
	  	~s,[\\\/],,;
	  	unshift(@RhapVERSIONS,$_);
	      }
	  }
	  foreach (@RhapVERSIONS) {
	    print "     find key ",$KeyToFind.$_,"\n";
	    my $Key =  $Registry->{$KeyToFind.$_};
	    if ($Key){
		  $InstallDir = $Key->{"InstallationDirectory"};
		  my $ud=$Key->{"UserDirectory"};
		  $root=$ud if($ud);
		  $InstallDir =~s,[\\\/]$,,;
		  print "     InstallationDirectory = $InstallDir\n";
		  $currentVersion=$_ if(-f $InstallDir."/rhapsody.exe");
		  last if(-f $InstallDir."/rhapsody.exe");
	    }

	  }
	}
	unless(-f $InstallDir."/rhapsody.exe"){
	    foreach(@InstallDIRS){
	        if(-f $_."/rhapsody.exe"){
		   $InstallDir=$_;
		   print "     InstallDir=$InstallDir\n";
		   last;
		} else {
		   my $dd=$_;
		   foreach (@RhapVERSIONS) {
		       if(-f $dd.'/'.$_."/rhapsody.exe"){
			   $InstallDir=$dd.'/'.$_;
			   $currentVersion=$_;
			   print "     InstallDir=$InstallDir\n";
			   last;
		       }
	           }
	        }
	    }
	    unless(-f $InstallDir."/rhapsody.exe"){
		print PerlTranslate("RhapInstall.pl", "[KO] Rhapsody installation not found. Restart with Rhapsody installation directory as parameter\n", "");
		return 1 ;
	    }
	}

    }
    #create executables
    copy ("$TESTRTDIR/bin/intel/win32/perllauncher.exe","$TESTRTDIR/bin/intel/win32/RhapTestRT.exe");
    copy ("$TESTRTDIR/bin/intel/win32/perllauncher.exe","$TESTRTDIR/bin/intel/win32/RhapTestRTreport.exe");
    if($currentVersion eq ''){
	print "[KO] The version of the used Rhapsody is not found in the installation.\n Please restart the installation with install dir as first arg and version in second arg\n";
    } else {
	my $v;
	if($currentVersion=~m/^(\d+)\.(\d+)/){
		my $Mv=$1;
		my $mv=$2;
		$v=$Mv.$mv;
		if($Mv == 7) {$v=76;}
	}
	unless(-f $FILESDIR."RhapConf$v.exe"){
		print "[KO] The version $v of the used Rhapsody is not known.\n Please contact the support to get appropriate RhapConf.exe\n";
	}
	print "[ok] copy ${FILESDIR}RhapConf$v.exe to $TESTRTDIR/bin/intel/win32/RhapConf.exe\n";
	copy($FILESDIR."RhapConf$v.exe","$TESTRTDIR/bin/intel/win32/RhapConf.exe");
    }
    if(-f "$TESTRTDIR/bin/intel/win32/RhapTestRT.exe"){
	print "[ok] RhapTestRT.exe and RhapTestRTreport.exe generated into $TESTRTDIR/bin/intel/win32\n";
    } else {
	print "[KO] It look like you have no permissions to make the installation.\n Please restart the installation as administrator\n";
	return 1;
    }
    #copy files into Rhapsody Install
    copy($FILESDIR."TestRealTime.xml",$InstallDir."/Doc/PropertyHelp/TestRealTime.xml");
    copy($FILESDIR."TestRealTime.xml",$InstallDir."/Doc/PropertyHelp/EN/TestRealTime.xml");
    if(-f $InstallDir."/Doc/PropertyHelp/TestRealTime.xml"){
	print "[ok] TestRealTime.xml copied into $InstallDir/Doc/PropertyHelp\n";
    } else {
	print "[KO] It look like you have no permissions to write into $InstallDir.\n Please restart the installation as administrator\n";
    }

    my $tm=insertToolMenu($InstallDir."/Rhapsody.ini");

    unless($root){
	$root=$InstallDir;
	$root=~s,^\w:\Program Files(\(x86\))?,$ENV{HOMEPATH},;
	$root =~s,[\\\/]$,,;
	unless(-f $root."/Rhapsody.ini"){
		$root=$InstallDir;
	    	$root=~s,^\w:\\Program Files(\(x86\))?,$ENV{USERPROFILE},; 
		$root =~s,[\\\/]$,,;
		unless(-f $root."/Rhapsody.ini"){
		    	$root.="x64"; 
		}
	}
    }
    $tm |=insertToolMenu($root."/Rhapsody.ini");
    unless($tm){
	print "[KO] Rhapsody.ini not found or no write permissions !!!\n";
    }
    if(-d $root."/Share"){
	copy($FILESDIR."TestRealTime.sbs",$root."/Share/Profiles/TestRealTime.sbs");
	copy($FILESDIR."TestRealTimeC.sbs",$root."/Share/Profiles/TestRealTimeC.sbs");
	if(-f $root."/Share/Profiles/TestRealTime.sbs"){
		print "[ok] sbs files copied into $root/Share/Profiles\n";
	} else {
		print "[KO] It look like you have no permissions to write into $root/Share/Profiles.\n Please restart the installation as administrator\n";
	}
	copy($FILESDIR."TestRT.bat",$root."/Share/etc/TestRT.bat");
	copy($FILESDIR."TestRTF.bat",$root."/Share/etc/TestRTF.bat");
	copyEnv2TDP($FILESDIR."Env2TDP.txt",$root."/Share/etc/Env2TDP.txt");
	if(-f $root."/Share/etc/TestRT.bat"){
		print "[ok] batch files copied into $root/Share/etc\n";
	} else {
		print "[KO] It look like you have no permissions to write into $root/Share/etc.\n Please restart the installation as administrator\n";
	}
	checkBuildBatch($root."/Share/etc/CygwinMake.bat","\%");
	checkBuildBatch($root."/Share/etc/msmake.bat","\%");
    } else {
	print "[KO] $root/Share directory not found !!!\n";
    }

    return 0;
}

sub copyEnv2TDP
{
#No more thing to do since 8.0.0.4 where a 'cvisual' TDP is automatically created, targetting the best cvisualX TDP version in accordance with the installed compiler
	my $src=shift;
	my $dest=shift;
#	print "     $src -> $dest\n";
	copy($src,$dest);
return;
	my $clVer=0;
	my $tmp=$ENV{TEMP};
	$tmp.="\\" if($tmp ne '');
	print PerlTranslate("RhapInstall.pl", "     Generating TDP correspondance file %1\n", "%1 is the generated file",$dest);
	system("cl 2>&1 >${tmp}TestRTRhapInstall.log");
	if(open (LOG, "${tmp}TestRTRhapInstall.log"))
	{
	    while(<LOG>){
		if(/Compiler\s+Version\s+(\d+)\./){
		    $clVer=$1 -6;
		    last;
		}
	    }
	    close LOG;
	}
	unlink("${tmp}TestRTRhapInstall.log");
	if($clVer >6){
	    if(open (IN, $src)){
		open (OU, ">$dest");
		while(<IN>){
		    if(/^Microsoft/){
			s,(ATLTGT=cvisual)6,$1$clVer,;
		    }
		}
		print OU;
		close IN;
	    }
	} else {
	    print "     Microsoft compiler not found => $dest unchanged\n";
	    copy($src,$dest);
	}
}


sub insertToolMenu
{
    my $src=shift;
    my $found=0;
    if(-f $src){
	print "     insert in ToolMenu $src....\n";
	if(open (IN, $src)){
	  if(open (OU, ">$src.tmp")){
	    while(<IN>){
		if(/numberOfElements=(\d+)/){
		    my $L=$1;
		    $found=1;
		    my $O=$L+1;
		    my $P=$O;
		    $O++;
		    my $Q=$O;
		    $O++;
		    my $M=$O;
		    $O++;
		    my $N=$O;
		    $O++;
		    s,$L,$O,;
		    print OU ;

		    print OU <<"END";
name$P=TestRealTime RuntimeAnalysis Enable
command$P=RhapConf.exe
arguments$P=-add "\$OMROOT\\Profiles\\TestRealTime.sbs" "\$OMROOT\\Profiles\\TestRealTimeC.sbs"
initialDir$P=
JavaMainClass$P=
JavaClassPath$P=
JavaLibPath$P=
isVisible$P=1
isMacro$P=0
isPlugin$P=0
isSynced$P=0
UsingJava$P=0
applicableTo$P=
applicableToProfile$P=
helperTriggers$P=
isPluginCommand$P=0
name$Q=TestRealTime RuntimeAnalysis Disable
command$Q=RhapConf.exe
arguments$Q=-remove "\$OMROOT\\Profiles\\TestRealTime.sbs" "\$OMROOT\\Profiles\\TestRealTimeC.sbs"
initialDir$Q=
JavaMainClass$Q=
JavaClassPath$Q=
JavaLibPath$Q=
isVisible$Q=1
isMacro$Q=0
isPlugin$Q=0
isSynced$Q=0
UsingJava$Q=0
applicableTo$Q=
applicableToProfile$Q=
helperTriggers$Q=
isPluginCommand$Q=0
name$M=TestRT results
command$M=RhapTestRTreport.exe
arguments$M=
initialDir$M=
JavaMainClass$M=
JavaClassPath$M=
JavaLibPath$M=
isVisible$M=1
isMacro$M=0
isPlugin$M=0
isSynced$M=0
UsingJava$M=0
applicableTo$M=
applicableToProfile$M=
helperTriggers$M=
isPluginCommand$M=0
name$N=TestRT clean results
command$N=RhapTestRTreport.exe
arguments$N=-clean
initialDir$N=
JavaMainClass$N=
JavaClassPath$N=
JavaLibPath$N=
isVisible$N=1
isMacro$N=0
isPlugin$N=0
isSynced$N=0
UsingJava$N=0
applicableTo$N=
applicableToProfile$N=
helperTriggers$N=
isPluginCommand$N=0
name$O=TestRT clean
command$O=RhapTestRTreport.exe
arguments$O=-cleanall
initialDir$O=
JavaMainClass$O=
JavaClassPath$O=
JavaLibPath$O=
isVisible$O=1
isMacro$O=0
isPlugin$O=0
isSynced$O=0
UsingJava$O=0
applicableTo$O=
applicableToProfile$O=
helperTriggers$O=
isPluginCommand$O=0
END
		} else {
		    print OU ;
		    if(/RhapTestRTreport/){$found=0;} #already there
		}
	    }
	    close IN;
	    close OU;
	  } else {
	    print "[KO] Unable to write on $src\n";
	    close IN;
	    return 0;
	  }
	}

	if($found){
	    rename("$src.tmp",$src);
	}else{
	    unlink("$src.tmp");
	}
	return 1;
    }
    return 0;
}

#check the build batch file
sub checkBuildBatch
{
my $makBatFile=shift;
my $prefix=shift;
my $modif=0;
    if(open(MAK,$makBatFile)){
	if(open(OUT,">$makBatFile.tmp")){
	    while(<MAK>){
		if(/make/ && /\Q$prefix\E1/){
		    if(/(\Q$prefix\E(\d))\s*$/){
		    	my $sp=$1;
		    	my $p=$2;
		    	my $i=9;
		    	if($p >2) {
			    while($p <$i){
				s,(\Q$sp\E),$1 $prefix$i,;
				$i--;
				$modif++;
			    }
			}
		    }
		}
		print OUT;
	    }
	    close OUT;
	    close MAK;
	    if($modif){
	        rename($makBatFile, $makBatFile.'.bak');
	        rename($makBatFile.'.tmp', $makBatFile);
	    } else {
	        unlink($makBatFile.'.tmp');
	    }
	}else {
	    print "[!!] Unable to write file $makBatFile\n";
	    close MAK;
        }
    } else {
	print "[!!] $makBatFile not found\n";
    }
}

exit(Install());
1;
