
#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

#No parameters are required except if the environment variable TESTRTDIR is not set.
my $silent=0;
$silent=1 if(@ARGV);#run it silently during install
my $TestRTDir=shift;
$TestRTDir=$ENV{TESTRTDIR} if($TestRTDir eq '');
die "Usage: postInstall.pl <TestRT Dir>\n" if($TestRTDir eq ""); # NO TRANSLATION

$SCRIPTDIR="$TestRTDir/lib/scripts/";
require $SCRIPTDIR . "Fcts.pl";
# No Bufferisation
$|=1;

my @xdpNative;
my @xdpGNU;
my @xdpFiles;

if($^O=~/win32/i) {
    $PATH_SEPARATOR = '\\';
    @xdpNative=qw(cvisual9 cvisual8 cvisual1X);
    @xdpNative=qw(cvisual9 cvisual8 cvisual_64 cvisual1X) if($ENV{PROCESSOR_ARCHITECTURE}=~m/64/);
    @xdpGNU=qw(cpcgnu64 cpcgnu cwinmingw3_2);
    @xdpFiles=qw(adapcgnt.xdp cwinvxsim.xdp cwinvxppc.xdp cwinVxRtpWB3.xdp cwindiabsgstep.xdp cwinmulti.xdp carm.xdp cCWHiWave9S08GT.xdp cwinccs5tms470.xdp);

  } else {
    $PATH_SEPARATOR = '/';
  }
$TARGETDIR=$TestRTDir.$PATH_SEPARATOR."targets".$PATH_SEPARATOR;
$XMLDIR =  $TestRTDir.$PATH_SEPARATOR."targets".$PATH_SEPARATOR."xml".$PATH_SEPARATOR;

if($silent){
	open(OLDOUT,">&STDOUT");
	open(OLDERR,">&STDERR");
	open(STDOUT,">$TestRTDir/targets/TDPInstall.log");
	open(STDERR,">&STDOUT");
}

#check the ctags installation
my $ctagsMissing=0;
my $status=system("ctags \"$TestRTDir\\lib\\SystemTesting\\template1.c\"");
if($status || !(-f "tags")) {
	print &PerlTranslate("postInstall.pl", "### Check ctags installation\n");
	print &PerlTranslate("postInstall.pl", "### ctags status=$status\n");
	#see if it exist into testRT installation
	$status=system("\"$TestRTDir/bin/intel/win32/ctags\" \"$TestRTDir\\lib\\SystemTesting\\template1.c\"");
	if($status || !(-f "tags")) {
		print &PerlTranslate("postInstall.pl", "*** TestRT ctags status=$status\n");
		$ctagsMissing=1;
		ctagsError();
	}
}
unlink("tags");

my $currentTDP='';
#generate native TDPs
NATIV:{
    foreach(@xdpNative)
    {
	~s/\.xdp$//;
    	$currentTDP=$_;
    	unless(-f $XMLDIR.$currentTDP.'.xdp')
    	{
	    print &PerlTranslate("postInstall.pl", "*** %1 not found\n", "%1 is the full name - with path - of the xdp file",$XMLDIR.$currentTDP.'.xdp');
	    next ;
	}
	if(generateTDP($currentTDP)) { last NATIV ; }
	print &PerlTranslate("postInstall.pl", "*** Generation of %1 failed\n", "%1 is the full name - with path - of the xdp file",$XMLDIR.$currentTDP.'.xdp');
    }
    if(@xdpNative){
	print &PerlTranslate("postInstall.pl", "*** All native TDP installation have failed\n");
	print &PerlTranslate("postInstall.pl", "*** This can be due to the development environment not being installed on this machine\n");
	print &PerlTranslate("postInstall.pl", "*** This TDP may not be usable\n\n");
    }
    $currentTDP='';
}
if($currentTDP ne ''){
    if($^O=~/win32/i) {
	# modify as Native TDP
	print &PerlTranslate("postInstall.pl", "*** copy of $currentTDP to cvisual and patch ini files\n");
	CopyDirRecurs($TARGETDIR.$currentTDP,$TARGETDIR.'cvisual');
	patchIniFile($TARGETDIR.'cvisual/tp.ini',"C Visual");
	patchIniFile($TARGETDIR.'cvisual/tpcpp.ini',"C++ Visual");
    } else {

    }
	sleep(1);
	print &PerlTranslate("postInstall.pl", "*** cvisual TDP has been installed based on $currentTDP.xdp\n");
}

#generate GNU TDPs
GNU:{
    foreach(@xdpGNU)
    {
	~s/\.xdp$//;
    	$currentTDP=$_;
	if(generateTDP($currentTDP)) { last GNU ; }
	print &PerlTranslate("postInstall.pl", "*** Generation of %1 failed\n", "%1 is the full name - with path - of the xdp file",$XMLDIR.$currentTDP.'.xdp');
    }
    if(@xdpGNU){
	print &PerlTranslate("postInstall.pl", "*** All GNU TDP installation have failed\n");
	print &PerlTranslate("postInstall.pl", "*** This can be due to the development environment not being installed on this machine\n");
	print &PerlTranslate("postInstall.pl", "*** This TDP may not be usable\n\n");
    }
    $currentTDP='';
}
if($currentTDP ne ''){
	# modify as Native TDP
	print &PerlTranslate("postInstall.pl", "*** copy of $currentTDP to cgnu and patch ini files\n");
	CopyDirRecurs($TARGETDIR.$currentTDP,$TARGETDIR.'cgnu');
	patchIniFile($TARGETDIR.'cgnu/tp.ini',"C GNU");
	patchIniFile($TARGETDIR.'cgnu/tpcpp.ini',"C++ GNU");
	sleep(1);
	print &PerlTranslate("postInstall.pl", "*** GNU TDP has been installed based on $currentTDP.xdp\n");
}

#generate other cross TDPs
foreach(@xdpFiles)
{
	~s/\.xdp$//;
    	$currentTDP=$_;
	unless(generateTDP($currentTDP)) {
		print &PerlTranslate("postInstall.pl", "*** Generation of %1 failed\n", "%1 is the full name - with path - of the xdp file",$XMLDIR.$currentTDP.'.xdp');
		print &PerlTranslate("postInstall.pl", "*** This can be due to the development environment not being installed on this machine\n");
		print &PerlTranslate("postInstall.pl", "*** This TDP may not be usable\n\n");
	}
}

if($silent){
	close(STDOUT);open(STDOUT,">&OLDOUT");
	close(STDERR);open(STDERR,">&OLDERR");

	close(OLDOUT);
	close(OLDERR);
}

if($ctagsMissing)
{
	ctagsError() ;
	sleep(15);
}
exit(0);

sub generateTDP
{
	my $xdpName=shift;
	print &PerlTranslate("postInstall.pl", "### Generating TDP folder %1 from %2\n", "%2 is the full name - with path -  of the xdp file and %1 is the target directory of the product delivery",$xdpName,$XMLDIR.$xdpName.'.xdp');
	print ("\"$TestRTDir/bin/intel/win32/TestRTperl.exe\" -I\"$TestRTDir/lib/perl\" \"$TestRTDir/lib/scripts/tdpgen.pl\" -check \"$TestRTDir/targets/xml/$xdpName.xdp\" $tdp\n");
	my $Status=system("\"$TestRTDir/bin/intel/win32/TestRTperl.exe\" -I\"$TestRTDir/lib/perl\" \"$TestRTDir/lib/scripts/tdpgen.pl\" -check \"$TestRTDir/targets/xml/$xdpName.xdp\" $tdp");
	return 1 if($Status==0 && ((-f $TARGETDIR.$xdpName.'/tp.ini') || (-f $TARGETDIR.$xdpName.'/ada95.ini')));
	return 0; # failed
}

sub patchIniFile
{
	# Patch the ini file.
	my $atl_cc_in = shift;
	my $name=shift;
	my $atl_cc_out = $atl_cc_in.".out";
	if(open (IN, "<$atl_cc_in")){
		open (OUT, ">$atl_cc_out") or print "Can't open \"$atl_cc_out\"\n";
		while (<IN>)
		{
			if(/^NAME=/) {
				print OUT "NAME=\"$name\"\n";
				~s/^NAME=/ORGNAME=/;
			}
			print OUT;
		}
		close (IN);
		close (OUT);
		# Rename the out file as the infile
		unlink ($atl_cc_in);
		rename ($atl_cc_out, $atl_cc_in);
	} else { print "Can't open \"$atl_cc_in\"\n"; }
}


sub ctagsError()
{
                               print "*********************************************************************************\n";
    print &PerlTranslate("postInstall.pl", "Unable to find ctags.exe please check your installation\n", "");
    print &PerlTranslate("postInstall.pl", "ctags.exe has to be downloaded from http://sourceforge.net/projects/ctags\n", "");
    print &PerlTranslate("postInstall.pl", "once downloaded copy it into %1\\bin\\intel\\win32\n", "%1 is the product install directory,$TestRTDir");
                                print "*********************************************************************************\n";

}

