##!perl -w
#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2011 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

# useful for stdout/stderr direct viewing inside 'console' of studio window
# (better: we have stdout/stderr lines in right order and not stderr before stdout
use FileHandle ;

STDOUT->autoflush(1);
STDERR->autoflush(1);

unless ($ENV{'TESTRTDIR'}) {
	print "Please set TESTRTDIR environment variable\n";
}
$SCRIPTDIR=$ENV{'TESTRTDIR'}.'/lib/scripts/';
# Miscellaneous functions
require $SCRIPTDIR . "Fcts.pl";

my $appName="com.ibm.rational.testrt.test.campaign.launcher.TestLaunchers";
my @STYLESHEETS = ("testsuite_report","coverage_report","mempro_report","perfpro_report");

my $workspace="";
my $args="";
my @testsuitelist=();
my $testsuites="";
my $build="";
my $buildArgs="";
my $reportdir="";
while(@ARGV) {
    $_=shift(@ARGV);

	if(/^-WORKSPACE=(.+)/i){
		$workspace=$1;
	}
	elsif (/^-BUILD_PROJECT=(.+)/i) {
		$build="-build ".$1." "."-cleanBuild ".$1." ";
		$buildArgs="-BUILD_PROJECT=".$1." ";
	}
	elsif (/^-REPORTDIR=(.+)/i) {
		$reportdir=$1;
		$args.="-xmlfile=\"$reportdir/Summary.xml\" -XMLREPORTS ";
	}
	elsif (/^-BINDIR=(.+)/i || /^-TDPDIR=(.+)/i) {
		$args.=$_." ";
	}
	else {
		push(@testsuitelist,$_);
	}
}

if ($workspace eq "") {
	my @dirs=split('[\\\/]',$testsuitelist[0]);
	$workspace=shift(@dirs);
	while(!(-d $workspace.'/.metadata') && @dirs){
		$workspace.='/'.shift(@dirs);
	}
	unless(@dirs) {
		print "WorkSpace Not found in $testsuitelist[0]\n" ;
		print "\nUsage rtrteclipse:\n";
		print "\t[-WORKSPACE={workspace directory}]\n";
		print "\t[testsuite_pathname [{testsuite_pathname}]]\n";
		print "\t[-BUILD_PROJECT={project_name | all}]\n";
		print "\t[-BINDIR={directory}]\n";
		print "\t[-TDPDIR={directory}]\n";
		print "\t[-REPORTDIR={directory}]\n"; #directory where to copy all xml report file
		print "\ne.g: rtrteclipse -WORKSPACE={workspace} {testsuitePathFromWorkspace} [{testsuite}] [options]\n";
		print "\ne.g: rtrteclipse {testsuiteWithAbsolutePath} [{testsuite}] [options]\n";

		exit(1);
	}
	print "WorkSpace found as $workspace because .metadata found there\n";
	$testsuitelist[0]='/'.join('/',@dirs);# make it relative to workspace
}
$workspace='"'.$workspace.'"' if($workspace=~/\s/);
my $tspath=$testsuitelist[0]; $tspath=~s,[\\\/][^\\\/]*$,,;#get the path to the first ts
foreach my $ts (@testsuitelist){
	unless(-f $workspace.'/'.$ts || -f $workspace.'/'.$ts.'.test_suite') {
		if(-f $ts || -f $ts.'.test_suite'){
			$ts=~s,^\Q$workspace\E,,;
		} else {
			# try to reuse the path from the first test_suit
			$ts=$tspath.'/'.$ts if(-f $workspace.'/'.$tspath.'/'.$ts || $workspace.'/'.$tspath.'/'.$ts.'.test_suite')
		}
	}
	$ts='"'.$ts.'"' if($ts=~/\s/);
	$testsuites.=$ts.' ';
}
if($reportdir){
    if(-d $reportdir){
	unlink("$reportdir/Summary.xml");
    } else {
	mkdir($reportdir);
    }
    open(LOG,">>$reportdir/rtrteclipse.log");
} else {
    open(LOG,">>rtrteclipse.log");
}

if ($testsuites eq "" and $build eq "") { # No args means build all
	$build="-build all -cleanBuild all ";
	$buildArgs="-BUILD_PROJECT=all ";
}

my $retBuild=0;
if (!($build eq "")) { # Build application
	my $buildCommand = "\"$ENV{'TESTRTDIR'}/eclipsec\" -nosplash -application org.eclipse.cdt.managedbuilder.core.headlessbuild -data $workspace $build $args";
	print $buildCommand,"\n";
	print LOG $buildCommand,"\n";
	$retBuild=system ($buildCommand);
	print LOG "status=$retBuild\n";
}

if ($retBuild==0) {
	my $command = "\"$ENV{'TESTRTDIR'}/eclipsec\" -nosplash -application $appName -data $workspace $testsuites $buildArgs $args";
	print "\n$command\n";
	print LOG "\n$command\n";
	$retBuild=system ($command);
	print "eclipse execution status=$retBuild\n";

	if($reportdir ne '') {
	    if(-f "$reportdir/Summary.xml")
	    {
		copyEclipseTestSuiteReports("$reportdir/Summary.xml",$reportdir,'');
		copy("$reportdir/Summary.xml","$reportdir/Summary_".time.".xml");
	    } else {
		$retBuild =1;# exit code 1 if no execution
		print "No execution report found\n";
		print LOG "No execution report found\n";
	    }
	}
}
#print join(',',@STYLESHEETS)," not copied\n";
print "exit status $retBuild\n";
close LOG;
exit($retBuild);

my $workspace_path;
sub copyEclipseTestSuiteReports
{
  my $resultSummaryFile=shift;
  my $destDir=shift;
  my $destResFile=shift;
  my $xslToBeCopied=1;
  my $translate=0;
  my $covTotal=0;
  my $covCovered=0;
  if(open(XML, "$resultSummaryFile") ) {
     if($destResFile ne '') {
	     if(open(OUT,">$destResFile")) {$translate=1;} else { print LOG "Unable to write file $destDir/$destResFile\n";}
     }
     while(<XML>){
	 if(/<RUN /){
		 my $logf;
		 if(/WORKSPACE_PATH=\"([^\"]*)\"/) {$workspace_path=$1;}
		 if(/LOG=\"([^\"]*)\"/) { $logf=$1;copyReport("/$1",$destDir); }
		 $translate && print OUT "<RUNS>\n<LOG>$destDir/$logf</LOG>\n";
		 next;
	 }
	 elsif(/<(TESTSUITECALL|QUANTIFY|PURIFY|TRACE|METRICS|CODEREVIEW)_RESULT\s/)
	 {
		 if(/REPORT_PATH=\"([^\"]*)\"/){copyReport($1,$destDir);}
	 }
	 elsif(/<COVERAGE_RESULT\s/)
	 {
		 if(/REPORT_PATH=\"([^\"]*)\"/){copyReport($1,$destDir);}
		 my $t=0;my $c=0;
		 if(/BRANCH_NUMBER=.(\d+). /){
			 $t=$1;
			 if(/COVERED_BRANCH=.(\d+). /){
				 $c=$1;
				 if($covTotal<$t){
					 $covTotal=$t;
					 $covCovered=$c;
				 } elsif($covTotal==$t && $covCovered<$c){
					 $covCovered=$c;
				 }
			 }
		 }
		 next;
	 }
	 elsif(/<\/COVERAGE_RESULT/)
	 {
		 next;
	 }
	 elsif(/<\/RUN>/)
	 {
		 $translate && print OUT "<COVERAGE_RESULTS COVERED_BRANCH=\"$covCovered\" BRANCH_NUMBER=\"$covTotal\" ></COVERAGE_RESULTS>\n";
	 }
	 #old report format kept for compatibility (before 8011)
         if(/<(REPORT|LOG)>(.+)<\/(REPORT|LOG)>/){
         	my ($d,$b,$e)=SplitFileName($2);
         	#copy($2,$destDir.'/'.$b.'.'.$e);

		# Copy reports
		my @REPORTSEXT = ("test_report","coverage_report","mempro_report","perfpro_report");
		foreach (@REPORTSEXT) {
			my $report_src = $d.'/'.$b.'.'.$_;
			my $report_dest = $destDir.'/'.$b.'.'.$_;

			if (-f $report_src) {
				copy($report_src, $report_dest);
				 print LOG "$report_src not found !!!\n" unless(-f $report_dest);
			}
		}

		# Copy style sheets if not already done
		my @STYLESHEETS = ("testsuite_report","coverage_report","mempro_report","perfpro_report");
		foreach (@STYLESHEETS) {
			my $styleSheet_src = $d.'/'.$_.'.xsl';
			my $styleSheet_dest = $destDir.'/'.$_.'.xsl';

			if (-f $styleSheet_src) {
				copy($styleSheet_src, $styleSheet_dest);
				 print LOG "$styleSheet_src not found !!!\n" unless(-f $styleSheet_dest);
			}
		}
         }

	 if($translate){
		 s,<TESTSUITECALL_RESULT ,<RUN ,;
		 s/\/RUN/\/RUNS/;
		 s/\/TESTSUITECALL_RESULT/\/RUN/;
        	 print OUT ;
	 }
     }
     close XML;
     close OUT if($translate);
  } else {
     print LOG "TestRTRqm.pl: Unable to read the results file $resultSummaryFile\n";
  }
}
my %stylesheetcopied;
sub copyReport
{
	my ($d,$b,$e)=SplitFileName(shift);
	my $destDir=shift;
	$d=$workspace_path.$d;
	my $report_src = $d.$b.'.'.$e;
	my $report_dest = $destDir.'/'.$b.'.'.$e;
	copy($report_src,$destDir);
	print LOG "$report_src not found !!!\n" unless(-f $report_dest);
	unless($stylesheetcopied{$e}){
		$stylesheetcopied{$e}=1;
		if($e=~m/_report$/){
			$e=~s,test_report,testsuite_report,;
			copy($d.$e.'.xsl',$destDir);
			print LOG "$e.xsl not found !!!\n" unless(-f $destDir.'/'.$e.'.xsl');
		}
	}
}
1;
